package ru.dbotthepony.mc.otm.block.entity;

import net.minecraft.core.BlockPos;
import net.minecraft.core.Direction;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.nbt.ListTag;
import net.minecraft.nbt.Tag;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.TranslatableComponent;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.inventory.AbstractContainerMenu;
import net.minecraft.world.item.Item;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.block.entity.BlockEntityType;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraftforge.common.capabilities.Capability;
import net.minecraftforge.common.util.LazyOptional;
import ru.dbotthepony.mc.otm.Registry;
import ru.dbotthepony.mc.otm.capability.IMatterGridCell;
import ru.dbotthepony.mc.otm.capability.IMatterTaskProvider;
import ru.dbotthepony.mc.otm.capability.MatterTask;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.matter.MatterGrid;
import ru.dbotthepony.mc.otm.menu.MatterPanelMenu;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.*;
import java.util.stream.Collectors;

public class BlockEntityMatterPanel extends BlockEntityMattery implements IMatterGridCell, IMatterTaskProvider {
	public BlockEntityMatterPanel(BlockPos p_155229_, BlockState p_155230_) {
		super(Registry.BlockEntities.MATTER_PANEL, p_155229_, p_155230_);
	}

	private static final TranslatableComponent NAME = new TranslatableComponent("block.overdrive_that_matters.matter_panel");

	@Override
	protected Component getDefaultDisplayName() {
		return NAME;
	}

	@Nullable
	@Override
	public AbstractContainerMenu createMenu(int containerID, Inventory inventory, Player ply) {
		return new MatterPanelMenu(containerID, inventory, this);
	}

	private boolean valid = true;

	@Override
	public void invalidateCaps() {
		super.invalidateCaps();
		valid = false;
	}

	@Override
	public void reviveCaps() {
		super.reviveCaps();
		valid = true;
	}

	private final LazyOptional<BlockEntityMatterPanel> resolver = LazyOptional.of(() -> this);

	@Nonnull
	@Override
	public <T> LazyOptional<T> getCapability(@Nonnull Capability<T> cap, @Nullable Direction side) {
		if (valid && (cap == MatteryCapability.MATTER_CELL || cap == MatteryCapability.TASK))
			return resolver.cast();

		return super.getCapability(cap, side);
	}

	private MatterGrid grid;

	@Override
	public void setLevel(Level p_155231_) {
		super.setLevel(p_155231_);

		if (grid == null)
			scheduleDiscoverNeighbours(getBlockPos(), p_155231_);
	}

	@Nullable
	@Override
	public MatterGrid getMatterGrid() {
		return grid;
	}

	@Override
	public boolean isValidMatterCell() {
		return valid;
	}

	@Override
	public void setMatterGrid(MatterGrid grid) {
		this.grid = grid;
	}

	@Nullable
	@Override
	public IMatterTaskProvider getTaskProvider() {
		return this;
	}

	private final HashMap<UUID, MatterTask> tasks = new HashMap<>();

	@Nonnull
	@Override
	public Collection<MatterTask> getTasks() {
		return tasks.values().stream().filter(task -> task.required() > 0).collect(Collectors.toList());
	}

	@Nonnull
	@Override
	public Collection<MatterTask> getAllTasks() {
		return List.copyOf(tasks.values());
	}

	@Nullable
	@Override
	public MatterTask allocateTask(boolean simulate) {
		for (var entry : tasks.entrySet()) {
			if (entry.getValue().required() > 0) {
				if (!simulate) {
					tasks.put(entry.getKey(), entry.getValue().shrinkRequired(1));
				}

				return entry.getValue();
			}
		}

		return null;
	}

	@Override
	public void notifyTaskCompletion(MatterTask task) {
		var get_task = tasks.get(task.id());

		if (get_task == null) {
			return;
		}

		get_task = get_task.shrinkInProgress(1);

		if (get_task.required() == 0 && get_task.in_progress() == 0) {
			tasks.remove(task.id());
		} else {
			tasks.put(task.id(), get_task);
		}
	}

	@Override
	public CompoundTag save(CompoundTag nbt) {
		ListTag list = new ListTag();

		for (MatterTask task : tasks.values()) {
			list.add(task.serializeNBT());
		}

		nbt.put("tasks", list);

		return super.save(nbt);
	}

	@Override
	public void load(CompoundTag nbt) {
		super.load(nbt);
		tasks.clear();

		ListTag list = nbt.getList("tasks", Tag.TAG_COMPOUND);

		for (Tag tag : list) {
			MatterTask task = MatterTask.deserializeNBT(tag);

			if (task != null) {
				tasks.put(task.id(), task);
			}
		}
	}

	@Nullable
	@Override
	public MatterTask getTask(UUID id) {
		return tasks.get(id);
	}

	public MatterTask addTask(Item item, int how_much) {
		var task = new MatterTask(UUID.randomUUID(), item, 0, 0, how_much);
		tasks.put(task.id(), task);
		return task;
	}

	@Override
	public void dropAllTasks() {
		tasks.clear();
	}
}
