package ru.dbotthepony.mc.otm.network;

import net.minecraft.network.FriendlyByteBuf;
import net.minecraftforge.api.distmarker.Dist;
import net.minecraftforge.fml.DistExecutor;
import net.minecraftforge.fmllegacy.network.NetworkEvent;
import ru.dbotthepony.mc.otm.capability.IPatternStorage;

import java.util.ArrayList;
import java.util.Collection;
import java.util.function.Supplier;

public class PatternStateSendListPacket {
	public PatternStateSendListPacket() {

	}

	public PatternStateSendListPacket(Collection<IPatternStorage.PatternState> states) {
		this.states.addAll(states);
	}

	public PatternStateSendListPacket(int container_id, Collection<IPatternStorage.PatternState> states) {
		this(states);
		this.container_id = container_id;
	}

	public int container_id;
	public ArrayList<IPatternStorage.PatternState> states = new ArrayList<>();

	public void encodeNetwork(FriendlyByteBuf buffer) {
		buffer.writeInt(container_id);
		buffer.writeInt(states.size());

		for (var state : states) {
			state.write(buffer);
		}
	}

	public void handleMessage(Supplier<NetworkEvent.Context> context) {
		context.get().setPacketHandled(true);

		context.get().enqueueWork(() -> {
			DistExecutor.unsafeRunWhenOn(Dist.CLIENT, () -> () -> ClientPatternStateSendListPacketHandler.handlePacket(this, context));
		});
	}

	public static PatternStateSendListPacket decodeNetwork(FriendlyByteBuf buffer) {
		var packet = new PatternStateSendListPacket();
		packet.container_id = buffer.readInt();
		int size = buffer.readInt();

		for (int i = 0; i < size; i++) {
			var state = IPatternStorage.PatternState.read(buffer);

			if (state != null) {
				packet.states.add(state);
			}
		}

		return packet;
	}
}
